# HexTools.py
# 1110-profs
# March 2, 2016
""" Graphics procedures that can be used to draw hexagons and
honeycombs.
"""

from SimpleGraphics import *
from math import sqrt


def DrawHexCell(a,b,FillColor=None):
    """
    Displays a regular hexagon with side one and center (a,b).
    The hexagon is colored according to FillColor.

    PreC: a and b are numbers, FillColor is an rgb list.
    """
    alfa = sqrt(3)/2
    x = [a+.5,    a-.5,   a-1,  a-.5 ,  a+.5,   a+1]
    y = [b+alfa,  b+alfa, b,    b-alfa, b-alfa, b]
    DrawPoly(x,y,FillColor,EdgeWidth=2,EdgeColor=RED)
    DrawDisk(a,b,.05,FillColor=BLACK)


def DrawHoneyComb(m):
    """ Draws a size-m honeycomb together with its
    honeycomb circle.

    Pre: m is positive integer that satisfies m>=3.
    """
    x = 0
    for i in range(0,m):
        if i%2==0:
            y = 0
        else:
            y = sqrt(3)/2
        for j in range(0,m):
            DrawHexCell(x,y,FillColor=YELLOW)
            DrawHexCell(x,-y,FillColor=YELLOW)
            DrawHexCell(-x,y,FillColor=YELLOW)
            DrawHexCell(-x,-y,FillColor=YELLOW)
            y = y + sqrt(3)
        x = x + 1.5
    DrawDisk(0,0,m-2,EdgeWidth=3,EdgeColor=BLACK)
    DrawHexCell(0,0,FillColor=PINK)
    s = 'DrawHoneyComb(%1d)' % m
    Title(s,FontSize=24)


def NeighborCenter(a,b,i):
    """ Returns floats u and v with the property that
        (u,v) is the center of neighbor i.

        Example. If someone made the following function call:

                  u,v = NeighborCenter(0.0,0.0,2)

        then u and v would respectively be assigned the x and y
        coordinates of the center of the second neighbor of the
        hex-cell centered at (0,0), i.e., u would be 1.5 and
        v would be sqrt(3)/2.

        PreC: a and b are floats and (a,b) is the center of a hex-cell.
        i is an int that satisfies 1<=i<=6.
    """
    delta1 = 1.5
    delta2 = sqrt(3)/2
    if i==1:
        u = a; v = b+2*delta2
    elif i==2:
        u = a+delta1; v = b+delta2
    elif i==3:
        u = a+delta1; v = b-delta2
    elif i==4:
        u = a; v = b-2*delta2
    elif i==5:
        u = a-delta1; v = b-delta2
    else:
        u = a-delta1; v = b+delta2
    return u,v


def DrawNeighbors(a,b):
    """ Draws a hex-cell at (a,b) and its six neighbors.
    PreC: a and b are floats.
    """

    # Draw the "central" hex cell...
    DrawHexCell(a,b,FillColor=YELLOW)
    # and its six neighbors
    for i in range(1,7):
        aNew,bNew = NeighborCenter(a,b,i)
        DrawHexCell(aNew,bNew,FillColor=CYAN)
    s = 'The Neighbors of DrawHexCell(%1d,%1d)' % (a,b)
    Title(s,FontSize=24)


def BeelineWorker():
    """ Returns a sample escape beelinw string for a size-15 honeycomb.
    """
    return '1313454253621451625316434631441635166244531314456513166561365655'


def BeelineDrone():
    """ Returns a sample escape beelins string for a size-15 honeycomb.
    """
    return '3z432z2z42z2z3z24zz3'


if __name__ == '__main__':

    # Draw a single hexagon...
    m = 2
    MakeWindow(m,bgcolor=WHITE,labels=True)
    DrawHexCell(0,0,FillColor=YELLOW)
    Title('DrawHexCell(0,0,FillColor=YELLOW)',FontSize=24)

    # Draw a honeycomb...
    m = 10
    MakeWindow(m,bgcolor=WHITE,labels=True)
    DrawHoneyComb(m)

    # Draw a hex-cell and its six neighbors...
    m = 3
    MakeWindow(m,bgcolor=WHITE,labels=True)
    DrawNeighbors(0,0)

    # Display all the windows...
    ShowWindow()

